/**
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:<br>
 * BitCtrl Systems GmbH<br>
 * Weißenfelser Straße 67<br>
 * 04229 Leipzig<br>
 * Phone: +49 341-490670<br>
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.emailfaxsms;

import java.util.Collection;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.kex.emailfaxsms.email.smtp.KexEmailSender;
import de.bsvrz.kex.emailfaxsms.email.texter.KexEmailTexter;

/**
 * Testet die Klasse KexEmailTexter
 *
 * @author BitCtrl Systems GmbH, Bachraty
 *
 */
public class KexEmailTexterTest extends KexEmailTexter {

	/**
	 * Testet den Versand, stat Absenden macht er eine TextAusgabe
	 *
	 * @author BitCtrl Systems GmbH, Bachraty
	 *
	 */
	protected class KexEmailSenderTest extends KexEmailSender {

		/**
		 * Standardkonstruktor
		 *
		 * @param mailServer
		 *            Addresse des EmailServers
		 * @param user
		 *            Username
		 * @param passwort
		 *            Passwort
		 * @param authReq
		 *            true, wenn Authentifizierung benoetigt ist
		 * @param starttls
		 *            true, wenn man den TLS Protokol benutzen soll
		 */
		public KexEmailSenderTest(final String mailServer, final String user, final String passwort,
				final boolean authReq, final boolean starttls) {
			super(mailServer, user, passwort, authReq, starttls);
		}

		@Override
		public VersandStatus send(final String absender, final String betreff, final String text,
				final String empfaenger, final Collection<String> ccList, final Collection<String> bccList) {

			System.out.print("# EML # VON:" + absender + " BETR:" + betreff + " TXT:" + text + " EMPF:" + empfaenger);
			System.out.print(" CC:");
			for (final String s : ccList) {
				System.out.print(s + ", ");
			}
			System.out.print(" BCC:");
			for (final String s : bccList) {
				System.out.print(s + ", ");
			}
			System.out.println("");
			try {
				Thread.sleep(1500);
			} catch (final Exception e) {
				e.printStackTrace();
			}
			return VersandStatus.VERSAND_OK;
		}
	}

	@Override
	public synchronized void update(final ResultData[] results) {
		super.update(results);

		for (final ResultData resDatei : results) {
			final Data data = resDatei.getData();
			if (data == null) {
				continue;
			}
			if (resDatei.getDataDescription().getAttributeGroup().equals(ddParam.getAttributeGroup())) {

				final String mailServer = data.getItem("MailServer").asTextValue().getText();
				final int authReq = data.getItem("Authentifizierung").asUnscaledValue().intValue();
				final String user = data.getItem("Benutzer").asTextValue().getText();
				final String pass = data.getItem("Passwort").asTextValue().getText();

				sender = new KexEmailSenderTest(mailServer, user, pass, (authReq > 0) ? true : false, false);
			}
		}
	}
}
