/*
 * Segment 2 (KEx),  SWE2.4 Email/Fax/SMS
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.emailfaxsms.email.texter;

import java.text.MessageFormat;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.kex.emailfaxsms.AbstraktKexVersandKomponente;
import de.bsvrz.kex.emailfaxsms.EmailFaxSmsMsg;
import de.bsvrz.kex.emailfaxsms.EmailVersandAuftrag;
import de.bsvrz.kex.emailfaxsms.IVersandAuftrag;
import de.bsvrz.kex.emailfaxsms.VersandStatus;
import de.bsvrz.kex.emailfaxsms.email.smtp.KexEmailSender;
import de.bsvrz.sys.funclib.bitctrl.daf.LogTools;

/**
 * Die Email Texter Klasse, erstellt eine E-Mail aus einem E-Mail Versandauftrag
 * und versendet diese mit Hilfe der EmailSender Klasse.
 *
 * @author BitCtrl Systems GmbH, Bachraty
 */
public class KexEmailTexter extends AbstraktKexVersandKomponente implements ClientReceiverInterface {

	/**
	 * Datenbeschreibung fuer die Parametrierung.
	 */
	protected DataDescription ddParam;
	/**
	 * StandardSender.
	 */
	private String stdSender;
	/**
	 * StandardEmpfaenger.
	 */
	private String stdEmpfaenger;
	/**
	 * StandardBetreff.
	 */
	private String stdBetreff;
	/**
	 * Die VersandKlasse.
	 */
	protected KexEmailSender sender;

	@Override
	public void init(final ClientDavInterface connection, final SystemObject modul, final int maxWarteschlangeGroesse) {
		super.init(connection, modul, maxWarteschlangeGroesse);

		ddParam = new DataDescription(getDav().getDataModel().getAttributeGroup("atg.versandModulExterneMeldungenEmail"),
				getDav().getDataModel().getAspect("asp.parameterSoll"));

		getDav().subscribeReceiver(this, modul, ddParam, ReceiveOptions.normal(), ReceiverRole.receiver());
	}

	@Override
	protected synchronized VersandStatus sendeAuftrag(final IVersandAuftrag auftrag, final int maxVersuche,
			final long pause) throws InterruptedException {

		// Warten auf die Parametrierung
		if (sender == null) {
			final String txt = MessageFormat.format(EmailFaxSmsMsg.WarningNochNichtParametriert.toString(),
					"E-Mail-Versand");
			LogTools.log(getLogger(), EmailFaxSmsMsg.WarningAllgemein, txt);
			return new VersandStatus(txt, false);
		}

		if (!(auftrag instanceof EmailVersandAuftrag)) {
			getLogger().warning("Email-Versand Komponente kann nur Email-Versand Aufträge bearbeiten");
			return VersandStatus.NICHT_VERSENDET;
		}

		final EmailVersandAuftrag eAuftr = (EmailVersandAuftrag) auftrag;

		String empf = eAuftr.getEmpfaenger();
		String betreff = eAuftr.getBetreff();
		String absender = eAuftr.getAbsender();

		if ((empf == null) || empf.equals("")) {
			empf = stdEmpfaenger;
		}
		if ((betreff == null) || betreff.equals("")) {
			betreff = stdBetreff;
		}
		if ((absender == null) || absender.equals("")) {
			absender = stdSender;
		}

		if ((empf == null) || (betreff == null) || (absender == null) || (eAuftr.getText() == null)
				|| empf.equals("")) {
			return new VersandStatus("Versandauftrag ist nicht komplett:" + eAuftr, false);
		}

		VersandStatus status = VersandStatus.NICHT_VERSENDET;

		for (int i = maxVersuche; i > 0; i--) {
			status = sender.send(absender, betreff, eAuftr.getText(), empf, eAuftr.getCCEmpfaenger(),
					eAuftr.getBCCEmpfaenger());
			if (status.getVersandErfolg()) {
				break;
			}
			Thread.sleep(pause);
		}
		return status;
	}

	@Override
	public synchronized void update(final ResultData[] results) {
		for (final ResultData resDatei : results) {
			final Data data = resDatei.getData();
			if (data == null) {
				continue;
			}
			if (resDatei.getDataDescription().getAttributeGroup().equals(ddParam.getAttributeGroup())) {

				LogTools.log(getLogger(), EmailFaxSmsMsg.InfoParameterEmpfangen, "E-Mail");

				final String mailServer = data.getItem("MailServer").asTextValue().getText();
				final int authReq = data.getItem("Authentifizierung").asUnscaledValue().intValue();
				final String user = data.getItem("Benutzer").asTextValue().getText();
				final String pass = data.getItem("Passwort").asTextValue().getText();

				if (sender != null) {
					sender = null;
				}
				sender = new KexEmailSender(mailServer, user, pass, (authReq > 0) ? true : false, true);

				stdSender = data.getItem("StandardSender").asTextValue().getText();
				stdEmpfaenger = data.getItem("StandardEmpfaenger").asTextValue().getText();
				stdBetreff = data.getItem("StandardBetreff").asTextValue().getText();

			}
		}
	}
}
