/*
 * Segment 2 (KEx),  SWE2.4 Email/Fax/SMS
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.emailfaxsms.sms.sender;

/**
 * Bildet Latin1-Codierung auf GSM ab.
 *
 * @author BitCtrl Systems GmbH, Bachraty
 */
public final class MapLatin1GSM {

	/**
	 * Nicht definierte Charaker weil ISO Latin1 8-bit und GSM 7-Bit ist,
	 * koennen nicht alle charakter Abbildet werden.
	 */
	protected static final char ND = 0x1b;

	/** Nicht definierte Charakter werden durch Null ersetzt (@ in GSM). */
	protected static final char NULL = 0x0;

	/** Space in GSM. */
	public static final char GSMSP = 0x20;

	/**
	 * Einige Latin(ASCII) charakter koennen als 2 byte charakter versendet
	 * werden (aber nur im PDU modus).
	 */
	protected static final char TC = 0x1b;

	/**
	 * Die Tabelle.
	 */
	protected static final char[] MAP_LATIN_GSM = new char[] { MapLatin1GSM.// GSM
																			// HEX,
																			// Latin
																			// HEX,
																			// Latin/HTML
																			// (
																			// wenn
																			// der
																			// Quellfile
																			// in
																			// Latin
																			// ist
																			// :)
	ND, // 0x00
		MapLatin1GSM.ND, // 0x01
		MapLatin1GSM.ND, // 0x02
		MapLatin1GSM.ND, // 0x03
		MapLatin1GSM.ND, // 0x04
		MapLatin1GSM.ND, // 0x05
		MapLatin1GSM.ND, // 0x06
		MapLatin1GSM.ND, // 0x07
		MapLatin1GSM.ND, // 0x08
		MapLatin1GSM.ND, // 0x09
		0x0A, // 0x0A <CR>
		MapLatin1GSM.ND, // 0x0B
		MapLatin1GSM.ND, // 0x0C
		0x0A, // 0x0D
		MapLatin1GSM.ND, // 0x0E
		MapLatin1GSM.ND, // 0x0F

		MapLatin1GSM.// 0x10, // 0x10 <LF>
		ND, // 0x10 <LF>
		MapLatin1GSM.ND, // 0x11
		MapLatin1GSM.ND, // 0x12
		MapLatin1GSM.ND, // 0x13
		MapLatin1GSM.ND, // 0x14
		MapLatin1GSM.ND, // 0x15
		MapLatin1GSM.ND, // 0x16
		MapLatin1GSM.ND, // 0x17
		MapLatin1GSM.ND, // 0x18
		MapLatin1GSM.ND, // 0x19
		MapLatin1GSM.ND, // 0x1A
		MapLatin1GSM.ND, // 0x1B
		MapLatin1GSM.ND, // 0x1C
		MapLatin1GSM.ND, // 0x1D
		MapLatin1GSM.ND, // 0x1E
		MapLatin1GSM.ND, // 0x1F

		0x20, // 0x20 <SP>
		0x21, // 0x21 !
		0x22, // 0x22 "
		0x23, // 0x23 #
		0x02, // 0x24 $
		0x25, // 0x25 %
		0x26, // 0x26 &
		0x27, // 0x27 '
		0x28, // 0x28 (
		0x29, // 0x29 )
		0x2A, // 0x2A *
		0x2B, // 0x2B +
		0x2C, // 0x2C ,
		0x2D, // 0x2D -
		0x2E, // 0x2E .
		0x2F, // 0x2F /

		0x30, // 0x30 0
		0x31, // 0x31 1
		0x32, // 0x32 2
		0x33, // 0x33 3
		0x34, // 0x34 4
		0x35, // 0x35 5
		0x36, // 0x36 6
		0x37, // 0x37 7
		0x38, // 0x38 8
		0x39, // 0x39 9
		0x3A, // 0x3A :
		0x3B, // 0x3B ;
		0x3C, // 0x3C <
		0x3D, // 0x3D =
		0x3E, // 0x3E >
		0x3F, // 0x3F ?

		0x00, // 0x40 @
		0x41, // 0x41 A
		0x42, // 0x42 B
		0x43, // 0x43 C
		0x44, // 0x44 D
		0x45, // 0x45 E
		0x46, // 0x46 F
		0x47, // 0x47 G
		0x48, // 0x48 H
		0x49, // 0x49 I
		0x4A, // 0x4A J
		0x4B, // 0x4B K
		0x4C, // 0x4C L
		0x4D, // 0x4D M
		0x4E, // 0x4E N
		0x4F, // 0x4F O

		0x50, // 0x50 P
		0x51, // 0x51 Q
		0x52, // 0x52 R
		0x53, // 0x53 S
		0x54, // 0x54 T
		0x55, // 0x55 U
		0x56, // 0x56 V
		0x57, // 0x57 W
		0x58, // 0x58 X
		0x59, // 0x59 Y
		0x5A, // 0x5A Z

		MapLatin1GSM.TC, // 0x5B [
		MapLatin1GSM.TC, // 0x5C \
		MapLatin1GSM.TC, // 0x5D ]
		MapLatin1GSM.TC, // 0x5E ^
		0x11, // 0x5F _
		MapLatin1GSM.ND, // 0x60 `
		0x61, // 0x61 a
		0x62, // 0x62 b
		0x63, // 0x63 c
		0x64, // 0x64 d
		0x65, // 0x65 e
		0x66, // 0x66 f
		0x67, // 0x67 g
		0x68, // 0x68 h
		0x69, // 0x69 i
		0x6A, // 0x6A j
		0x6B, // 0x6B k
		0x6C, // 0x6C l
		0x6D, // 0x6D m
		0x6E, // 0x6E n
		0x6F, // 0x6F o

		0x70, // 0x70 p
		0x71, // 0x71 q
		0x72, // 0x72 r
		0x73, // 0x73 s
		0x74, // 0x74 t
		0x75, // 0x75 u
		0x76, // 0x76 v
		0x77, // 0x77 w
		0x78, // 0x78 x
		0x79, // 0x79 y
		0x7A, // 0x7A z
		MapLatin1GSM.TC, // 0x7B {
		MapLatin1GSM.TC, // 0x7C |
		MapLatin1GSM.TC, // 0x7D }
		MapLatin1GSM.TC, // 0x7E ~
		MapLatin1GSM.ND, // 0x7F <DEL>

		MapLatin1GSM.ND, // 0x80 ND
		MapLatin1GSM.ND, // 0x81 ND
		MapLatin1GSM.ND, // 0x82 ND
		MapLatin1GSM.ND, // 0x83 ND
		MapLatin1GSM.ND, // 0x84 ND
		MapLatin1GSM.ND, // 0x85 ND
		MapLatin1GSM.ND, // 0x86 ND
		MapLatin1GSM.ND, // 0x87 ND
		MapLatin1GSM.ND, // 0x88 ND
		MapLatin1GSM.ND, // 0x89 ND
		MapLatin1GSM.ND, // 0x8A ND
		MapLatin1GSM.ND, // 0x8B ND
		MapLatin1GSM.ND, // 0x8C ND
		MapLatin1GSM.ND, // 0x8D ND
		MapLatin1GSM.ND, // 0x8E ND
		MapLatin1GSM.ND, // 0x8F ND

		MapLatin1GSM.ND, // 0x90 ND
		MapLatin1GSM.ND, // 0x91 ND
		MapLatin1GSM.ND, // 0x92 ND
		MapLatin1GSM.ND, // 0x93 ND
		MapLatin1GSM.ND, // 0x94 ND
		MapLatin1GSM.ND, // 0x95 ND
		MapLatin1GSM.ND, // 0x96 ND
		MapLatin1GSM.ND, // 0x97 ND
		MapLatin1GSM.ND, // 0x98 ND
		MapLatin1GSM.ND, // 0x99 ND
		MapLatin1GSM.ND, // 0x9A ND
		MapLatin1GSM.ND, // 0x9B ND
		MapLatin1GSM.ND, // 0x9C ND
		MapLatin1GSM.ND, // 0x9D ND
		MapLatin1GSM.ND, // 0x9E ND
		MapLatin1GSM.ND, // 0x9F ND

		0x20, // 0xA0 &NBSP (SPACE)
		0x40, // 0xA1 ! (upside down)
		MapLatin1GSM.ND, // 0xA2 &cent;
		0x01, // 0xA3 &pound;
		0x24, // 0xA4 &curren;
		0x03, // 0xA5 &yen;
		MapLatin1GSM.ND, // 0xA6 &brvbar;
		0x5F, // 0xA7 &sect;
		MapLatin1GSM.ND, // 0xA8 &uml;
		MapLatin1GSM.ND, // 0xA9 &copy;
		MapLatin1GSM.ND, // 0xAA &ordf;
		MapLatin1GSM.ND, // 0xAB &laqno;
		MapLatin1GSM.ND, // 0xAC &not;
		MapLatin1GSM.ND, // 0xAD &shy;
		MapLatin1GSM.ND, // 0xAE &reg;
		MapLatin1GSM.ND, // 0xAF &macr;

		MapLatin1GSM.ND, // 0xB0 &deg;
		MapLatin1GSM.ND, // 0xB1 &plusmn;
		MapLatin1GSM.ND, // 0xB2 &sup2;
		MapLatin1GSM.ND, // 0xB3 &sup3;
		MapLatin1GSM.ND, // 0xB4 &acute;
		MapLatin1GSM.ND, // 0xB5 &micro;
		MapLatin1GSM.ND, // 0xB6 &para;
		MapLatin1GSM.ND, // 0xB7 &middot;
		MapLatin1GSM.ND, // 0xB8 &cedil;
		MapLatin1GSM.ND, // 0xB9 &sup1;
		MapLatin1GSM.ND, // 0xBA &ordm;
		MapLatin1GSM.ND, // 0xBB &raquo;
		MapLatin1GSM.ND, // 0xBC &frac14;
		MapLatin1GSM.ND, // 0xBD &frac12;
		MapLatin1GSM.ND, // 0xBE &frac34;
		0x60, // 0xBF &iquest;

		MapLatin1GSM.ND, // 0xC0 &Agrave;
		MapLatin1GSM.ND, // 0xC1 &Aacute;
		0x0E, // 0xC2 &Acirc;
		MapLatin1GSM.ND, // 0xC3 &Atilde;
		0x5B, // 0xC4 &Auml;
		0x0E, // 0xC5 &Aring;
		0x1C, // 0xC6 &AElig;
		0x09, // 0xC7 &Ccedil;
		MapLatin1GSM.ND, // 0xC8 &Egrave;
		0x1F, // 0xC9 &Eacute;
		0x40, // 0xCA &Ecirc;
		MapLatin1GSM.ND, // 0xCB &Euml;
		MapLatin1GSM.ND, // 0xCC &Igrave;
		MapLatin1GSM.ND, // 0xCD &Iacute;
		MapLatin1GSM.ND, // 0xCE &Icirc;
		MapLatin1GSM.ND, // 0xCF &Iuml;

		MapLatin1GSM.ND, // 0xD0 &ETH;
		0x5D, // 0xD1 &Ntilde;
		MapLatin1GSM.ND, // 0xD2 &Ograve;
		MapLatin1GSM.ND, // 0xD3 &Oacute;
		MapLatin1GSM.ND, // 0xD4 &Ocirc;
		MapLatin1GSM.ND, // 0xD5 &Otilde;
		0x5C, // 0xD6 &Ouml;
		MapLatin1GSM.ND, // 0xD7 &times;
		0x0B, // 0xD8 &Oslash;
		MapLatin1GSM.ND, // 0xD9 &Ugrave;
		MapLatin1GSM.ND, // 0xDA &Uacute;
		MapLatin1GSM.ND, // 0xDB &Ucirc;
		0x5E, // 0xDC &Uuml;
		MapLatin1GSM.ND, // 0xDD &Yacute;
		MapLatin1GSM.ND, // 0xDE &THORN;
		0x1E, // 0xDF &szlig;

		0x7F, // 0xE0 &agrave;
		MapLatin1GSM.ND, // 0xE1 &aacute;
		MapLatin1GSM.ND, // 0xE2 &acirc;
		MapLatin1GSM.ND, // 0xE3 &atilde;
		0x7B, // 0xE4 &auml;
		0x0F, // 0xE5 &aring;
		0x1D, // 0xE6 &aelig;
		MapLatin1GSM.ND, // 0xE7 &ccedil;
		0x04, // 0xE8 &egrave;
		0x05, // 0xE9 &eacute;
		MapLatin1GSM.ND, // 0xEA &ecirc;
		MapLatin1GSM.ND, // 0xEB &euml;
		MapLatin1GSM.ND, // 0xEC &igrave;
		MapLatin1GSM.ND, // 0xED &iacute;
		MapLatin1GSM.ND, // 0xEE &icirc;
		MapLatin1GSM.ND, // 0xEF &iuml;

		MapLatin1GSM.ND, // 0xF0 &eth;
		0x7D, // 0xF1 &ntilde;
		0x08, // 0xF2 &ograve;
		MapLatin1GSM.ND, // 0xF3 &oacute;
		MapLatin1GSM.ND, // 0xF4 &ocirc;
		MapLatin1GSM.ND, // 0xF5 &otilde;
		0x7C, // 0xF6 &ouml;
		MapLatin1GSM.ND, // 0xF7 &divide;
		0x0C, // 0xF8 &oslash;
		0x06, // 0xF9 &ugrave;
		MapLatin1GSM.ND, // 0xFA &uacute;
		MapLatin1GSM.ND, // 0xFB &ucirc;
		0x7E, // 0xFC &uuml;
		MapLatin1GSM.ND, // 0xFD &yacute;
		MapLatin1GSM.ND, // 0xFE &thorn;
		MapLatin1GSM.ND, // 0xFF &yuml;
	};

	/**
	 * Abbildet eine Zeichenkette in Latin ins GSM fuer Text-Modus.
	 *
	 * @param str
	 *            Zeichenkette Latin
	 * @return Zeichenkette GSM
	 */
	public static String mapTexModus(final String str) {

		// ersetze "\r\n" (von Windoof) durch "\n"
		final String korrigiert = str.replaceAll("\r\n", "\n");

		final StringBuffer strBuffer = new StringBuffer(korrigiert.length() * 2);
		final char[] s = korrigiert.toCharArray();
		for (final char element : s) {
			final char c = MapLatin1GSM.MAP_LATIN_GSM[(element)];
			switch (c) {
			case ND:
				strBuffer.append(MapLatin1GSM.NULL);
				break;
			default:
				strBuffer.append(c);
			}
		}
		return strBuffer.toString();
	}

	/*
	 * 2 byte Character - Diese koennen nur im PDU Modus gesendet werden, weil
	 * <ESC> bricht im Textmodus den Versand ab
	 *
	 * CHAR DEC HEX GSM GSM HEX [ 91 0x5B <ESC> < 0x1B 0x3C \ 92 0x5C <ESC> /
	 * 0x1B 0x2F ] 93 0x5D <ESC> > 0x1B 0x3E ^ 94 0x5E <ESC> ^ 0x1B 0x14 { 123
	 * 0x7B <ESC> ( 0x1B 0x28 | 124 0x7C <ESC> @ 0x1B 0x40 } 125 0x7A <ESC> )
	 * 0x1B 0x29 ~ 126 0x7E <ESC> = 0x1B 0x3D euro <ESC> e
	 */

	/**
	 * Privater Konstruktor.
	 */
	private MapLatin1GSM() {
		/** nur zur Beruhigung von CheckStyle */
	}
}
