/*
 * Segment 2 (KEx),  SWE2.4 Email/Fax/SMS
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.emailfaxsms.vew;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.kex.emailfaxsms.AbstraktKexVersandKomponente;
import de.bsvrz.kex.emailfaxsms.EmailFaxSmsMsg;
import de.bsvrz.kex.emailfaxsms.EmailVersandAuftrag;
import de.bsvrz.kex.emailfaxsms.FaxVersandAuftrag;
import de.bsvrz.kex.emailfaxsms.IVersandAuftrag;
import de.bsvrz.kex.emailfaxsms.SmsVersandAuftrag;
import de.bsvrz.kex.emailfaxsms.SprachAnsageAuftrag;
import de.bsvrz.kex.emailfaxsms.VersandStatusSender;
import de.bsvrz.kex.emailfaxsms.VoIPFaxVersandAuftrag;
import de.bsvrz.kex.emailfaxsms.email.texter.KexEmailTexter;
import de.bsvrz.kex.emailfaxsms.fax.texter.KexFaxTexter;
import de.bsvrz.kex.emailfaxsms.sms.texter.KexSmsTexter;
import de.bsvrz.kex.emailfaxsms.voice.texter.KexVoiceTexter;
import de.bsvrz.kex.emailfaxsms.voipfax.texter.KexVoIPFaxTexter;
import de.bsvrz.sys.funclib.application.StandardApplication;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.bitctrl.daf.LogTools;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList.Argument;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.operatingMessage.MessageSender;

/**
 * Verwaltungsmodul der SWE 2.4. Steuert die Submodule f&uuml;r E-Mail, Fax und
 * SMS.
 *
 * @author BitCtrl Systems GmbH, Bachraty, Gieseler
 * @version $Id: KexEmailFaxSMS.java 63604 2016-09-15 11:49:19Z uhlmann $
 */
public class KexEmailFaxSMS implements ClientReceiverInterface, ClientSenderInterface, StandardApplication {

	/**
	 * Aufrufparameter Versandmodul.
	 */
	private static final String AP_MODUL = "-EmailFaxSMS";
	/**
	 * Aufrufparameter max. Warteschlangengroesse.
	 */
	private static final String AP_MAX_WARTESCHLANGE = "-maxGroesseWarteschlangen";
	/**
	 * Aufrufparameter Warngrenze Warteschlange.
	 */
	private static final String AP_WARN_WARTESCHLANGE = "-warnGroesseWarteschlangen";
	/**
	 * Aufrufparameter langeSMSzerbrechen.
	 */
	private static final String AP_SMSZERBRECHEN = "-langeSMSzerbrechen";

	/**
	 * Standardwert max. Warteschlangengroesse.
	 */
	private static final int DEFAULT_MAX_WARTESCHLANGE = 100;

	/**
	 * Standardwert Warngrenze Warteschlange.
	 */
	private static final int DEFAULT_WARN_WARTESCHLANGE = 20;

	/**
	 * Debug logger.
	 */
	private static Debug logger = Debug.getLogger();
	/**
	 * Verbindung zum DAV.
	 */
	public static ClientDavInterface dav;

	/**
	 * Die Hauptmethode.
	 *
	 * @param argumente
	 *            Aufrufparameter
	 */
	public static void main(final String[] argumente) {
		final KexEmailFaxSMS efs = new KexEmailFaxSMS();
		StandardApplicationRunner.run(efs, argumente);
	}

	/**
	 * Max Groesse der Warteschlangen.
	 */
	protected int maxGroesseWarteSchlangen = KexEmailFaxSMS.DEFAULT_MAX_WARTESCHLANGE;

	/**
	 * Lange SMS Zerbrechen.
	 */
	boolean langeSMSzerbrechen = false;

	/**
	 * Warnung Groesse der Warteschlangen.
	 */
	private int warnGroesseWarteSchlangen = KexEmailFaxSMS.DEFAULT_WARN_WARTESCHLANGE;

	/**
	 * Pid des Versandmodules, wenn als Parameter eingegeben.
	 */
	protected String modulPID;
	/**
	 * Id des Versandmodules, wenn als Parameter eingegeben.
	 */
	protected long modulID;

	/**
	 * Datenbeschreibung der empfangenen Email-Auftraege.
	 */
	protected DataDescription ddVERSANDAUFTRAGMAIL;

	/**
	 * Datenbeschreibung der empfangenen Fax-Auftraege.
	 */
	protected DataDescription ddVERSANDAUFTRAGFAX;

	/**
	 * Datenbeschreibung der empfangenen SMS-Auftraege.
	 */
	protected DataDescription ddVERSANDAUFTRAGSMS;

	/**
	 * Datenbeschreibung der empfangenen Sprachansage-Auftraege.
	 */
	private DataDescription ddVERSANDAUFTRAGSPRACHE;
	
	/**
	 * Datenbeschreibung der empfangenen VoIP-Fax-Auftraege.
	 */
	private DataDescription ddVERSANDAUFTRAGVOIPFAX;

	/**
	 * Datenbeschreibung der Parametrierung.
	 */
	protected DataDescription ddPARAMETRIERUNG;

	/**
	 * Versandkomponente FAX.
	 */
	protected AbstraktKexVersandKomponente kompFax;

	/**
	 * Versandkomponente Email.
	 */
	protected AbstraktKexVersandKomponente kompEmail;

	/**
	 * Versandkomponente SMS.
	 */
	protected AbstraktKexVersandKomponente kompSms;
	private KexVoiceTexter kompSprache;
	private KexVoIPFaxTexter kompVoIPFax;

	/**
	 * Bearbeitet einen FaxVersandAuftrag.
	 *
	 * @param data
	 *            Datensatz des Auftrags
	 */
	public void bearbeiteFaxAuftrag(final Data data) {
		final IVersandAuftrag auftrag = new FaxVersandAuftrag();
		auftrag.init(data);
		final int groesse = kompFax.bearbeite(auftrag);
		if (groesse > maxGroesseWarteSchlangen) {
			LogTools.log(KexEmailFaxSMS.logger, EmailFaxSmsMsg.WarningWarteSchlangeVoll, "Fax");
		} else if ((groesse * 100) >= (maxGroesseWarteSchlangen * warnGroesseWarteSchlangen)) {
			LogTools.log(KexEmailFaxSMS.logger, EmailFaxSmsMsg.WarningWarteschlangeFuellgrad, "Fax",
					warnGroesseWarteSchlangen);
		}
	}

	/**
	 * Bearbeitet einen EmailVersandAuftrag.
	 *
	 * @param data
	 *            Datensatz des Auftrags
	 */
	public void bearbeiteMailAuftrag(final Data data) {

		final IVersandAuftrag auftrag = new EmailVersandAuftrag();
		auftrag.init(data);
		final int groesse = kompEmail.bearbeite(auftrag);
		if (groesse > maxGroesseWarteSchlangen) {
			LogTools.log(KexEmailFaxSMS.logger, EmailFaxSmsMsg.WarningWarteSchlangeVoll, "E-Mail");
		} else if ((groesse * 100) >= (maxGroesseWarteSchlangen * warnGroesseWarteSchlangen)) {
			LogTools.log(KexEmailFaxSMS.logger, EmailFaxSmsMsg.WarningWarteschlangeFuellgrad, "E-Mail",
					warnGroesseWarteSchlangen);
		}
	}

	/**
	 * Bearbeitet die Parametrierung des Verwaltungmoduls.
	 *
	 * @param data
	 *            Datensatz der Parametrierung
	 */
	public void bearbeiteParametrierung(final Data data) {
		int nVersuche;
		long pause;

		nVersuche = (int) data.getItem("WiederholungsVersuche").asUnscaledValue().longValue();
		pause = data.getItem("WiederholungsPause").asUnscaledValue().longValue();

		kompEmail.parametriereGenerell(nVersuche, pause);
		kompFax.parametriereGenerell(nVersuche, pause);
		kompSms.parametriereGenerell(nVersuche, pause);
		kompSprache.parametriereGenerell(nVersuche, pause);
		kompVoIPFax.parametriereGenerell(nVersuche, pause);
	}

	/**
	 * Bearbeitet einen SMSVersandAuftrag.
	 *
	 * @param data
	 *            Datensatz des Auftrags
	 */
	public void bearbeiteSmsAuftrag(final Data data) {

		final IVersandAuftrag auftrag = new SmsVersandAuftrag();
		auftrag.init(data);
		final int groesse = kompSms.bearbeite(auftrag);
		if (groesse > maxGroesseWarteSchlangen) {
			LogTools.log(KexEmailFaxSMS.logger, EmailFaxSmsMsg.WarningWarteSchlangeVoll, "SMS");
		} else if ((groesse * 100) >= (maxGroesseWarteSchlangen * warnGroesseWarteSchlangen)) {
			LogTools.log(KexEmailFaxSMS.logger, EmailFaxSmsMsg.WarningWarteschlangeFuellgrad, "SMS",
					warnGroesseWarteSchlangen);
		}
	}

	/**
	 * Bearbeitet einen SprachAnsageAuftrag.
	 *
	 * @param data
	 *            Datensatz des Auftrags
	 */
	public void bearbeiteSprachAuftrag(final Data data) {

		final IVersandAuftrag auftrag = new SprachAnsageAuftrag();
		auftrag.init(data);
		final int groesse = kompSprache.bearbeite(auftrag);
		if (groesse > maxGroesseWarteSchlangen) {
			LogTools.log(KexEmailFaxSMS.logger, EmailFaxSmsMsg.WarningWarteSchlangeVoll, "Sprache");
		} else if ((groesse * 100) >= (maxGroesseWarteSchlangen * warnGroesseWarteSchlangen)) {
			LogTools.log(KexEmailFaxSMS.logger, EmailFaxSmsMsg.WarningWarteschlangeFuellgrad, "Sprache",
					warnGroesseWarteSchlangen);
		}
	}
	
	/**
	 * Bearbeitet einen VoIP-Fax-Auftrag.
	 *
	 * @param data
	 *            Datensatz des Auftrags
	 */
	public void bearbeiteVoIPAuftrag(final Data data) {

		final IVersandAuftrag auftrag = new VoIPFaxVersandAuftrag();
		auftrag.init(data);
		final int groesse = kompVoIPFax.bearbeite(auftrag);
		if (groesse > maxGroesseWarteSchlangen) {
			LogTools.log(KexEmailFaxSMS.logger, EmailFaxSmsMsg.WarningWarteSchlangeVoll, "VoIP-Fax");
		} else if ((groesse * 100) >= (maxGroesseWarteSchlangen * warnGroesseWarteSchlangen)) {
			LogTools.log(KexEmailFaxSMS.logger, EmailFaxSmsMsg.WarningWarteschlangeFuellgrad, "VoIP-Fax",
					warnGroesseWarteSchlangen);
		}
	}

	/**
	 * Beenden des Moduls.
	 */
	public void beenden() {
		kompEmail.terminate();
		kompFax.terminate();
		kompSms.terminate();
		kompSprache.terminate();
		kompVoIPFax.terminate();
	}

	@Override
	public void dataRequest(final SystemObject object, final DataDescription dataDescription, final byte state) {
		// TODO Auto-generated method stub

	}

	@Override
	public void initialize(final ClientDavInterface connection) throws Exception {
		KexEmailFaxSMS.dav = connection;
		KexEmailFaxSMS.logger = Debug.getLogger();
		final StringBuilder b = new StringBuilder(KexEmailFaxSMS.class.getSimpleName());
		final String incarnationName = connection.getClientDavParameters().getIncarnationName();
		if (incarnationName != null && !incarnationName.isEmpty()) {
			b.append(": ");
			b.append(incarnationName);
		}
		MessageSender.getInstance().setApplicationLabel(b.toString());
		ddPARAMETRIERUNG = new DataDescription(KexEmailFaxSMS.dav.getDataModel().getAttributeGroup(
				"atg.versandModulExterneMeldungen"), KexEmailFaxSMS.dav.getDataModel().getAspect("asp.parameterSoll"));

		ddVERSANDAUFTRAGFAX = new DataDescription(KexEmailFaxSMS.dav.getDataModel().getAttributeGroup(
				"atg.faxVersandAuftrag"), KexEmailFaxSMS.dav.getDataModel().getAspect("asp.auftrag"));

		ddVERSANDAUFTRAGMAIL = new DataDescription(KexEmailFaxSMS.dav.getDataModel().getAttributeGroup(
				"atg.emailVersandAuftrag"), KexEmailFaxSMS.dav.getDataModel().getAspect("asp.auftrag"));

		ddVERSANDAUFTRAGSMS = new DataDescription(KexEmailFaxSMS.dav.getDataModel().getAttributeGroup(
				"atg.smsVersandAuftrag"), KexEmailFaxSMS.dav.getDataModel().getAspect("asp.auftrag"));

		ddVERSANDAUFTRAGSPRACHE = new DataDescription(KexEmailFaxSMS.dav.getDataModel().getAttributeGroup(
				KexVoiceTexter.VERSANDAUFTRAG_ATG), KexEmailFaxSMS.dav.getDataModel().getAspect("asp.auftrag"));
		
		ddVERSANDAUFTRAGVOIPFAX = new DataDescription(KexEmailFaxSMS.dav.getDataModel().getAttributeGroup(
				KexVoIPFaxTexter.VERSANDAUFTRAG_ATG), KexEmailFaxSMS.dav.getDataModel().getAspect("asp.auftrag"));

		SystemObject vmem;
		if (modulID != 0) {
			vmem = KexEmailFaxSMS.dav.getDataModel().getObject(modulID);
		} else {
			vmem = KexEmailFaxSMS.dav.getDataModel().getObject(modulPID);
		}

		if (vmem == null) {
			throw new Exception("Versandmodul " + ((modulID != 0) ? modulID : modulPID) + " wurde nicht gefunden: ");
		}

		try {
			VersandStatusSender.getInstanz().init(KexEmailFaxSMS.dav, vmem);

			kompEmail = new KexEmailTexter();
			kompFax = new KexFaxTexter();
			kompSms = new KexSmsTexter(langeSMSzerbrechen);
			kompSprache = new KexVoiceTexter();
			kompVoIPFax = new KexVoIPFaxTexter();

			kompEmail.init(KexEmailFaxSMS.dav, vmem, maxGroesseWarteSchlangen);
			kompFax.init(KexEmailFaxSMS.dav, vmem, maxGroesseWarteSchlangen);
			kompSms.init(KexEmailFaxSMS.dav, vmem, maxGroesseWarteSchlangen);
			kompSprache.init(KexEmailFaxSMS.dav, vmem, maxGroesseWarteSchlangen);
			kompVoIPFax.init(KexEmailFaxSMS.dav, vmem, maxGroesseWarteSchlangen);

			kompEmail.start();
			kompFax.start();
			kompSms.start();
			kompSprache.start();
			kompVoIPFax.start();

			LogTools.log(KexEmailFaxSMS.logger, EmailFaxSmsMsg.FineAlleThreadsGestartet);

			KexEmailFaxSMS.dav.subscribeReceiver(this, vmem, ddPARAMETRIERUNG, ReceiveOptions.normal(),
					ReceiverRole.receiver());

			KexEmailFaxSMS.dav.subscribeReceiver(this, vmem, ddVERSANDAUFTRAGFAX, ReceiveOptions.normal(),
					ReceiverRole.drain());
			KexEmailFaxSMS.dav.subscribeReceiver(this, vmem, ddVERSANDAUFTRAGMAIL, ReceiveOptions.normal(),
					ReceiverRole.drain());
			KexEmailFaxSMS.dav.subscribeReceiver(this, vmem, ddVERSANDAUFTRAGSMS, ReceiveOptions.normal(),
					ReceiverRole.drain());
			KexEmailFaxSMS.dav.subscribeReceiver(this, vmem, ddVERSANDAUFTRAGSPRACHE, ReceiveOptions.normal(),
					ReceiverRole.drain());
			KexEmailFaxSMS.dav.subscribeReceiver(this, vmem, ddVERSANDAUFTRAGVOIPFAX, ReceiveOptions.normal(),
					ReceiverRole.drain());
			LogTools.log(KexEmailFaxSMS.logger, EmailFaxSmsMsg.FineVerwaltungKomplett);
		} catch (final Exception e) {
			e.printStackTrace();
		}

	}

	@Override
	public boolean isRequestSupported(final SystemObject object, final DataDescription dataDescription) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public void parseArguments(final ArgumentList argumente) throws Exception {
		// Debug.init("SWE 2.4 E-Mail/Fax/SMS", argumente);

		final String versandModul = argumente.fetchArgument(KexEmailFaxSMS.AP_MODUL).asNonEmptyString();

		if (versandModul.matches("\\d+")) {
			modulID = Long.parseLong(versandModul);
		} else {
			modulPID = versandModul;
		}

		if (argumente.hasArgument(KexEmailFaxSMS.AP_MAX_WARTESCHLANGE)) {
			final String str = argumente.fetchArgument(KexEmailFaxSMS.AP_MAX_WARTESCHLANGE).asString();
			if (str.matches("\\d+")) {
				maxGroesseWarteSchlangen = Integer.parseInt(str);
			} else {
				throw new Exception("MaxGroesseWarteschlangen kann nicht geparst werden: " + str);
			}
		}

		if (argumente.hasArgument(KexEmailFaxSMS.AP_WARN_WARTESCHLANGE)) {
			final String str = argumente.fetchArgument(KexEmailFaxSMS.AP_WARN_WARTESCHLANGE).asString();
			if (str.matches("\\d+")) {
				warnGroesseWarteSchlangen = Integer.parseInt(str);
				if ((warnGroesseWarteSchlangen < 1) || (warnGroesseWarteSchlangen > 100)) {
					throw new Exception("WarnGroesseWarteschlangen muss zwischen 1% und 100% liegen");
				}
			} else {
				throw new Exception("WarnGroesseWarteschlangen kann nicht geparst werden: " + str);
			}
		}

		if (argumente.hasArgument(KexEmailFaxSMS.AP_SMSZERBRECHEN)) {
			final Argument argParameter = argumente.fetchArgument(KexEmailFaxSMS.AP_SMSZERBRECHEN);
			if (!argParameter.hasValue()) {
				langeSMSzerbrechen = true;
			} else {
				langeSMSzerbrechen = Boolean.parseBoolean(argParameter.asString());
			}
		}

		argumente.ensureAllArgumentsUsed();
	}

	@Override
	public void update(final ResultData[] results) {
		for (final ResultData resDatei : results) {
			final Data data = resDatei.getData();
			if (data == null) {
				continue;
			}

			if (resDatei.getDataDescription().getAttributeGroup().equals(ddPARAMETRIERUNG.getAttributeGroup())) {
				LogTools.log(KexEmailFaxSMS.logger, EmailFaxSmsMsg.FineParametrierungEmpfangen);
				bearbeiteParametrierung(data);
			}
			if (resDatei.getDataDescription().getAttributeGroup().equals(ddVERSANDAUFTRAGFAX.getAttributeGroup())) {
				LogTools.log(KexEmailFaxSMS.logger, EmailFaxSmsMsg.InfoAuftragEmpfangen, "Fax");
				bearbeiteFaxAuftrag(data);
			}

			if (resDatei.getDataDescription().getAttributeGroup().equals(ddVERSANDAUFTRAGMAIL.getAttributeGroup())) {
				LogTools.log(KexEmailFaxSMS.logger, EmailFaxSmsMsg.InfoAuftragEmpfangen, "E-Mail");
				bearbeiteMailAuftrag(data);
			}

			if (resDatei.getDataDescription().getAttributeGroup().equals(ddVERSANDAUFTRAGSMS.getAttributeGroup())) {
				LogTools.log(KexEmailFaxSMS.logger, EmailFaxSmsMsg.InfoAuftragEmpfangen, "SMS");
				bearbeiteSmsAuftrag(data);
			}

			if (resDatei.getDataDescription().getAttributeGroup().equals(ddVERSANDAUFTRAGSPRACHE.getAttributeGroup())) {
				LogTools.log(KexEmailFaxSMS.logger, EmailFaxSmsMsg.InfoAuftragEmpfangen, "Sprache");
				bearbeiteSprachAuftrag(data);
			}

			if (resDatei.getDataDescription().getAttributeGroup().equals(ddVERSANDAUFTRAGVOIPFAX.getAttributeGroup())) {
				LogTools.log(KexEmailFaxSMS.logger, EmailFaxSmsMsg.InfoAuftragEmpfangen, "VoIP-Fax");
				bearbeiteVoIPAuftrag(data);
			}

		}
	}

}
